"use strict";

var KEY_TEXT = "text";
var KEY_CONTAINER = "container";
var fonts = ["", "null", undefined];
var splitter = null;
if (typeof(GraphemeSplitter) != "undefined") {
  splitter = new GraphemeSplitter();
}
String.prototype.replaceAll = function(f, e) {
  var reg = new RegExp(f, "g");
  return this.replace(reg, e);
};

function update(model) {
  try{
  var data = decodeURIComponent(model);
  console.log("update "+ data);
  var hasFont = true;
  var textObject = JSON.parse(data.replaceAll("\n", "\\n"));
  // var textModel = Object.assign(new TextModel(), textObject);
  var textModel =  deepClone(textObject, []);
  var texts = document.getElementsByClassName(KEY_TEXT);

  for (var i = 0; i < texts.length; i++) {
    var text = document.getElementById(KEY_TEXT + "_" + i);
    if (text === null) {
      continue;
    }
    if (textModel[i].text != "") {
      var textContent = textModel[i].text.replaceAll("\n", "<br>");
      if (splitter) {
        text.innerHTML = '';
        var textArr = splitter.splitGraphemes(textContent) || [];
        textArr.forEach(function(item) {
          text.innerHTML += isEmojiCharacter(item) ? "<span>"+item+"</span>" : "<span class='textGradient_" + i + "'>"+item+"</span>";;
        });
      } else {
        text.innerHTML = textContent;
      } 
    }
    text.style.fontFamily = textModel[i].fontFamily;
    if (textModel[i].fontFamily != "" && textModel[i].fontFamily != "null") {
      text.style.fontFamily = textModel[i].fontFamily;
    }
    if (fonts.indexOf(textModel[i].fontUrl) === -1) {
      addFont(textModel[i].fontFamily, textModel[i].fontUrl);
      hasFont = false;
    }
  }

  if (hasFont) {
    sendResponse();
  }
  } catch(errorMsg){
    jsCallback.onResponseError("error is "+errorMsg+" model = "+model);
  }
}
function isEmojiCharacter(string) {
  return string.toString().length > 1;
}

function addFont(fontFamily, fontUrl) {
  fonts.push(fontUrl);
  var fontStyle = document.createElement("style");
  fontStyle.appendChild(
    document.createTextNode(
      "@font-face {\
      font-family: " +
        fontFamily +
        ";\
      src: url('" +
        fontUrl +
        "');}"
    )
  );
  document.head.appendChild(fontStyle);
  WebFont.load({
    custom: {
      families: [fontFamily]
    },
    active: function() {
      console.log("font loaded...");
      sendResponse();
    }
  });
}

function getTextModel() {
  var result = [];
  var texts = document.getElementsByClassName(KEY_TEXT);
  for (var i = 0; i < texts.length; i++) {
    var textModel = new TextModel();
    var text = document.getElementById(KEY_TEXT + "_" + i);
    if (text === null) {
      continue;
    }
    textModel.text = text.innerHTML.replaceAll("<br>", "\n");
    if (typeof text.style.fontFamily === "undefined") {
      textModel.fontFamily = "";
    } else {
      textModel.fontFamily = text.style.fontFamily;
    }
    result.push(textModel);
  }
  return result;
}

window.onload = function() {
  if (typeof location.search != "undefined" && location.search.length > 1) {
    var data = location.search.substring(1);
    if (data.length > 1) {
      data = data.split("=")[1];
      update(data);
      return;
    }
  }
  console.log("onload");
  sendResponse();
};

function sendResponse() {
  console.log("sendResponse....");
  var response = new Response();
  response.textModel = getTextModel();
  getImageRect(response);
  var jsonString = JSON.stringify(response);
  console.log(jsonString);
  jsCallback.onResponse(jsonString);
}

function getImageRect(response) {
  var texts = document.getElementsByClassName(KEY_TEXT);
  var rect = [Number.POSITIVE_INFINITY, Number.POSITIVE_INFINITY, 0, 0];
  for (var i = 0; i < texts.length; i++) {
    var text = document.getElementsByClassName(KEY_CONTAINER + "_" + i)[0];
    var r = text.getBoundingClientRect();
    if (r.left < rect[0]) {
      rect[0] = r.left;
    }

    if (r.top < rect[1]) {
      rect[1] = r.top;
    }

    if (r.right > rect[2]) {
      rect[2] = r.right;
    }

    if (r.bottom > rect[3]) {
      rect[3] = r.bottom;
    }
  }
  response.imageLeft = rect[0];
  response.imageTop = rect[1];
  response.imageRight = rect[2];
  response.imageBottom = rect[3];
}

function deepClone(sourceObj, targetObj) {
  var cloneObj = targetObj || {};
  if (
    !sourceObj ||
    typeof sourceObj !== "object" ||
    sourceObj.length === undefined
  ) {
    return sourceObj;
  }
  if (sourceObj instanceof Array) {
    cloneObj = sourceObj.concat();
  } else {
    for (var i in sourceObj) {
      if (typeof sourceObj[i] === "object") {
        cloneObj[i] = deepClone(sourceObj[i], {});
      } else {
        cloneObj[i] = sourceObj[i];
      }
    }
  }
  return cloneObj;
}

function Response(imageLeft, imageTop, imageRight, imageBottom, textModel) {
    this.imageLeft = imageLeft;
    this.imageTop = imageTop;
    this.imageRight = imageRight;
    this.imageBottom = imageBottom;
    this.textModel = textModel;
}

function TextModel(text, fontFamily, fontUrl) {
    this.text = text;
    this.fontFamily = fontFamily;
    this.fontUrl = fontUrl;
}
